// Simple caps plug-in support example on Win32, should fit most users
//
// www.caps-project.org

#include "stdafx.h"

#include "ComType.h"
#include "CapsAPI.h"

#include "windows.h"
#include "CapsPlug.h"



int main(int argc, char* argv[])
{
	// load the plug-in, if available
	// if plug-in is not available or certain functions are not supported
	// affected calls return with an error
	CapsInit("CAPSImg.dll");

	// create image container
	// NOTE: this function fails with a negative result code, not with imge...
	// Image containers are like virtual drives
	// It is a good idea to add as many as you'd like to use:
	// e.g. add as many "floppy drives" as your emulated system allows
	// Once you have the containers use the Image Lock/Unlock functions
	int img=CapsAddImage();

	// lock this image into the container
	CapsLockImage(img, "test.ipf");

	// get some information about the image
	CapsImageInfo ci;
	CapsGetImageInfo(&ci, img);

	// just some helper to show the intended platform's name
	PCHAR format=CapsGetPlatformName(ci.platform[0]);

	// simplest use, let the decoder take care of cell mapping and data generation
	// align is set as well, it is only required for UAE usage
	// NOTE: you must unlock a track if you wish to change the locking method
	// if you don't unlock, the involved functions return with the cache
	UDWORD flag=DI_LOCK_ALIGN|DI_LOCK_DENVAR|DI_LOCK_DENAUTO|DI_LOCK_DENNOISE|DI_LOCK_NOISE;

	// decode and pre-cache the whole image, in other words lock each track
	// NOTE: not mandatory, but track locking on locked tracks return their cache,
	// therefore changing a track does not cost execution time
	CapsLoadImage(img, flag);

	// decode and cache some tracks.
	// NOTE: subsequent calls requesting the same track return the cache;
	// it's free in execution time
	// in this example LoadImage already cached all the tracks
	// you must unlock a track if you want to change the flags associated with it
	// each track can have a different flag
	// remember LoadImage locks all the tracks, with the flag given that time
	CapsTrackInfo ti;
	CapsLockTrack(&ti, img, 0, 0, flag);

	// ... do something...

	CapsLockTrack(&ti, img, 0, 1, flag);

	// ... do something...

	CapsLockTrack(&ti, img, 1, 0, flag);

	// ... do something...
	CapsLockTrack(&ti, img, 1, 1, flag);

	// ... do something...
	// ETC

	// LoadImage already locked the track, so INDEX flag is ignored
	CapsLockTrack(&ti, img, 0, 0, flag|DI_LOCK_INDEX);

	// you don't want to unlock a track unless desperate for memory or
	// to change the decoder flag
	CapsUnlockTrack(img, 0, 0);

	// see the difference, now INDEX flag is not ignored
	CapsLockTrack(&ti, img, 0, 0, flag|DI_LOCK_INDEX);

	// for convenience it is possible to unlock all tracks, which 
	// compliments nicely with LoadImage
	// It is not mandatory to unlock tracks:
	// locking a new image, unlocking an image or removing a container
	// do all necessary cleanup
	CapsUnlockAllTracks(img);

	// remove image from container
	// NOTE: not mandatory, locking another image does an unlock first
	CapsUnlockImage(img);

	// destroy image container
	// NOTE: this function fails with a negative result code, not with imge...
	CapsRemImage(img);

	// remove the library, and invalidate its entries.
	// Does destroy all the containers
	CapsExit();
	return 0;
}
